.TH ANN_FF ANN "May 2001" "HRy" "ANN Toolbox"
.so ../sci.an
.SH NAME
ANN_FF - Algorithms for feedforward nets.
.SH OBJECTIVE
To provide engines for feedforward ANN exploration,
testing and rapid prototyping.
.LP
Some flexibility is provided (e.g. the possibility to change the
activation or error functions).
.SH ARCHITECTURE DESCRIPTION
.TP
(a)
The network is visualized as follows: inputs at the left and data
(signals) propagating to the right.
.TP
(b)
N is a row vector containing the number of neurons per layer,
input included.
.TP
(c)
First layer is input (despite the fact that it does not process data
it makes implementation clearer).
.nf
                    Layer no.
            1    2    ...   size(N,'c')
        
            . -- o             o ->
              \\/
              /\\
        i   . -- o             o ->   o
        n     \\           |\\          u
        p      \\     =====| >         t
        u                 |/          p
        t                             u
        s      /                      t
              /                       s
             . - o             o ->
           input first        output
                 hidden

.fi
Note that connections do not jump over layers, they are only between
adjacent layers (fully interconnected).
.TP
(d)
The dimension of N is size(N,'c') so:
.RS
.TP 3
-
input layer have N(1) neurons
.TP 3
-
first hidden layer have N(2) neurons, ...
.TP 3
-
the output layer L have N(size(N,'c')) neurons
.RE
.TP
(e)
The input vector/matrix is x, each pattern is represented by one column.
Only constant size input patterns are accepted.

NOTE: Internally the patterns will be worked with, individually,
as column vectors, i.e. each pattern vector is a column of the form:
x(:,p), (p being the pattern order number).
.TP
(f)
Each neuron on first hidden layer have N(1) inputs, ...
for layer l in [2, ..., size(N,'c')] each neuron have N(l-1) inputs
from previous layer plus one simulating the bias (where applicable,
most algorithms assume existence of bias).
.TP
(g)
The network is fully connected but a connection can be canceled by
zeroing the corresponding weight (note that a training procedure may
turn it back to a non-zero value, this is one reason for which some
"hooks" are provided, see "ex" parameter below).
.SH USER INTERFACE (1): PARAMETERS
This subsection describes the parameters taken by the various functions
defined within the toolbox, not all functions require all parameters.
In alphabetical order:
.TP
af
gives activation function and, if required, its (total) derivative.
It is either:
.RS
.TP 3
-
a string giving the name of activation function
.TP 3
-
a two element row vector of strings where af(1) is
the string with the name of activation function
and af(2) is the name of the derivative.
.LP
NOTE: Given an activation function y = f(x), the derivative
have to be expressed in terms of y not x).
E.g. given the logistic activation function:
.nf
                1
        y = -----------
            1 + exp(-x)
.fi
the derivative will be expressed as:
.nf
        dy
        -- = y (1 - y)
        dx
.fi
This form reduces the memory usage and, in the particular case
of the logistic activation function, increases speed as well.

This parameter is optional, default value is either
"ann_log_activ" or ["ann_log_activ","ann_d_log_activ"]
(depending on the function using it),
i.e. the activation function, described above, and its derivative
(the default functions are already defined in this toolbox).

WARNING: Be very careful how to define a new activation function.  These
functions accept patterns as column vectors within y, each element
representing the total input on a neuron, and return a similar matrix
representing the activation of the whole layer. I.e. the logistic is
defined as:
.nf
	
        y = 1 ./ (1 + %e^(-x))

.fi
and note the space between 1 and ./
The derivative of activation function is defined similarly:
.nf

        z = y .* (1 - y)

.fi
and note the ".*" operator.
.RE
.TP
Delta_W_old
The quantity by which W was changed on previous training pattern.
.TP
dW, dW2
the amount of variations of each W element for calculating the error
derivatives trough a finite difference approach (see ann_FF_grad and
ann_FF_Hess for more information).
.TP
ef
error function.

This parameter is optional, default value is "ann_sum_of_sqr",
i.e. the sum-of-squares (already defined within this toolbox).
.TP
err_deriv_y
the error derivative with respect to network outputs. Returns a matrix
each column containing the error derivative corresponding to
the appropriate pattern.

This parameter is optional, default value is "ann_d_sum_of_sqr"
(already defined within this toolbox), i.e. the derivative of
sum-of-squares error function.
.TP
ex
is a Scilab program sequence, executed after the weight hypermatrix for
each training pattern have been updated. Its main purpose is to provide
hooks in order to change the learning function without having to rewrite it.
Typical usages would be: checking for a stop criteria, pruning.

This parameter is optional, default value is [" "] or [" "," "]
(some functions have two hooks), i.e. empty string, does nothing.
.TP
l
range of layers between which the network is run. Two component row vector:
.RS
.TP
l(1)
layer into which a pattern will be injected, presented as it
would have come from previous layer: l(1)-1.
.TP
l(2)
layer from which the outputs are collected. E.g.:
.RS
.TP
l = [3,3]
means input is injected into neurons from layer 3 and their
outputs (l(2)=3) are collected to give the result.
.TP
l = [2,3]
means input is injected into first hidden layer (exactly as it would
have come from input layer) and output is collected at the outputs of
neurons on layer 3.
.RE

This parameter is optional, default value is [2,size(N,'c')] (whole network).

WARNING: l(1) = 1 does not make sense as it represents the input layer.
.RE
.TP
lp
represents the learning parameters, is a row vector [lp(1), lp(2), ...]
The actual significance of each component may vary, see the respective
man pages for representation and typical values.
.TP
N
row vector, defines the network, i.e. no. of neurons per layer.
N(l) represents the number of neurons on layer l. E.g.:
.RS
.TP
N(1)
is the size of input vector,
.TP
N(size(N),'c')
is the size of output vector
.RE
.TP
r
range of random numbers based on which the connection weights (not biases)
are initialized. Is a two component row vector:
.RS
.TP
r(1)
gives the lower limit
.TP
r(2)
gives the upper limit
.LP
This parameter is optional, default value is [-1,1].
.RE
.TP
rb
range of random numbers based on which the biases (not other weihts)
are initialized. Is a two component row vector:
.RS
.TP
rb(1)
gives the lower limit
.TP
rb(2)
gives the upper limit
.LP
This parameter is optional, default value is [0,0], i.e. biases are
initialized with 0.
.RE
.TP
t
matrix of targets, one pattern per column. E.g. t(:,p) represents pattern
no. p.
.TP
x
matrix of inputs, one pattern per column. E.g. x(:,p) represents pattern
no. p
.SH USER INTERFACE (2): FUNCTIONS
The function names are built as follows:
.TP
ann
prefix for all function names within this toolbox.
_FF
prefix for all function names designed for feedforward nets.
.TP_online_, _batch_
defines the type of algorithm: online uses one pattern at a time,
batch uses all patterns at once.
.TP
_nb
postfix for all function names within this toolbox designed for
networks without bias.
.TP
ann_FF_init
Build and initialize the weight hypermatrix.
.TP
ann_FF_Std
Standard (vanilla, delta rule) backpropagation algorithm.
.TP
ann_FF_Mom
Backpropagation with momentum.
.TP
ann_FF_run
Runs the network.
.TP
ann_FF_grad
Calculate the error gradient trough a finite difference approach.
It is provided for testing purposes only.
.TP
ann_FF_Jacobian
Calculate the Jacobian trough a finite difference approach.
It is provided for testing purposes only.
.TP
ann_FF_Jacobian_BP
Calculate the Jacobian trough a backpropagation algorithm.
.TP
ann_FF_Hess
Calculate the Hessian trough a finite difference approach.
It is provided for testing purposes only.
.TP
ann_FF_VHess
Calculate the multiplication between a vector and the Hessian
trough an efficient finite difference approach.
.TP
ann_FF_ConjugGrad
Conjugate gradients algorithm.
.TP
ann_FF_SSAB
Backpropagation with SuperSAB algorithm.
.SH TIPS AND TRICKS
.TP 3
-
Do not use the no-bias networks unless you know what you are doing.
.TP 3
-
The most efficient (by far) algorithm is the "Conjugate Gradient",
however it may require bootstrapping with another algorithm
(see the examples).
.TP 3
-
Reduce as much is possible the number of loops and the number of
function calls, use instead as much is possible the matrix
manipulation capabilities of Scilab.
.TP 3
-
You can do a shuffling of training patterns between two calls to the
training procedure, use the "ex" hooks provided.
.TP 3
-
Be very careful when defining new activation and error functions and
test them to make sure they do what are supposed to do.
.TP 3
-
don't use sparse matrices unless they are really sparse (< 5%).
.SH IMPLEMENTATION DETAILS
.TP 3
-
Each layer have associated a hypermatrix of weights.

NOTE: Most algorithms assume existence of bias by default.

For each layer l, except l=1, the weight matrix associated with
connections from l-1 to l is W(1:N(l),1:N(l-1),l-1) for networks
without biases and W(1:N(l),1:N(l-1)+1,l-1) for networks with biases,
i.e. biases are stored in first column: W(1:N(l),1,l-1).
The total input to a layer l is:
.RS
.TP 3
=
W(1:N(l),1:N(l-1),l-1)*z(1:N(l-1)) for network without biases
.TP 3
=
W(1:N(l),1:N(l-1)+1,l-1)*[1; z(1:N(l-1))] for network with biases
.LP
where z(1:N(l-1)) is output of previous layer (column vector), i.e. bias is
simulated as neuron no. 0 on each layer with constant output 1.
.LP
W is initialized to:
.TP
hypermat(max(N),max(N),size(N,'c')-1)
for networks without biases,
.TP
"hypermat(max(N),max(N)+1,size(N,'c')-1)"
for networks with biases;
.LP
the unused entries from W are initialized to
zero and left untouched.
.RE
.TP 3
-
Pattern vectors are passed as columns in a matrix representing
a set (of patterns).
.SH NOTES
.TP 3
-
No sanity checks are performed as this will greatly hurt speed.
It is assumed that (as least to some extent) you know
what you are doing ;-) You can implement them yourself if you wish.

The following conditions have to be met:
.RS
.TP 3
+
targets have to have the same size as output layer,
i.e. size(target,'r') = N(size(N,'c'))
.TP 3
+
inputs have to have the same size as input layer,
i.e. size(input,'r') = N(1)
.TP 3
+
all N(i) have to be positive integers of course (am I paranoid here ? :-)
.TP 3
+
lp parameter is a row-vector of numbers
(actual dimension depends on the algorithm used).
.TP 3
+
af is a row-vector of string(s)
defining a valid activation function (and its derivative)
as appropriate for the algorithm used.
.TP 3
+
ex is a string or a two row-vector of strings, representing
valid Scilab set of instructions.
.TP 3
+
l(1) <= l(2) (see definition of l above) and l(1) >= 2
.TP 3
+
r(1) <= r(2) and rb(1) <= rb(2) (see definition of r and rb above),
if not then the program may run but you may not get what you would
expect when initializing the weights. Warning: In some particular
cases this may lead to very subtle errors (e.g. your program may
even run without generating any Scilab errors but the results may be
meaningless).
.RE
.TP 3
-
The algorithms themselfs are not described here, there are many books which
describes them (e.g. get mine "Matrix ANN" wherever you may find it ;-).
.TP 3
-
Hypermatrices are slow, however there is no other reasonable way of
doing things; tests performed by myself show that using embedded
matrices may increase speed but the manipulation of submatrices "by
hand" is very tedious and error prone. Of course you may rewrite the
algorithms for yourself using embedded matrices if you want to. If you
really need speed then go directly to C++ or whatever.
.SH SEE ALSO
.TP
ANN
overview of ANN toolbox.
.TP
ANN_GEN
overview of support functions.
.TP
ANN_FF_INT
description of implementation internals.
.TP
ann_FF_*
all functions related to feedforward nets.
.SH AUTHOR
Ryurick M. Hristev <r.hristev@phys.canterbury.ac.nz>
.SH COPYING
Copyright (C) Ryurick M. Hristev

The ANN Toolbox is licensed under GNU Public Licence version 2.
