.TH ann_FF_ConjugGrad ANN "May 2001" "HRy" "ANN Toolbox"
.so ../sci.an
.SH NAME
ann_FF_ConjugGrad - Conjugate Gradient algorithm.
.SH CALLING SEQUENCE
W = ann_BP_ConjugGrad(x,t,N,W,T,dW[,ex,af,err_deriv_y])
.SH PARAMETERS
.TP
W
The weight hypermatrix; it have to be initialized with ann_BP_init function.
.TP
x
Matrix of input patterns, one pattern per column.
.TP
t
Matrix of target patterns, one pattern per column.
.TP
N
Row vector describing the number of neurons per layer.
N(1) is the size of input pattern vector, N(size(N,'c')) is the size
of output pattern vector (and also target).
.TP
T
Number of training cycles (epochs, discrete time, steps).
.TP
dW
The quantity used to calculate the product between a direction and the
Hessian trough a finite differences algorithm; this parameter is
passed to ann_BP_VHess function (see its man page for details).
.TP
ex
string representing a valid Scilab sequence. It is executed after
the weight hypermatrix have been updated (i.e. after each epoch), using
"execstr".

This parameter is optional, default value: " "
(empty string, i.e. does nothing).
.TP
af
The name of activation function to be used (string).
.sp
This parameter is optional, default value "ann_log_activ",
i.e. the logistic activation function.
.TP
err_deriv_y
The name of error function derivative with respect to network outputs
(string).

This parameter is optional, default value is "ann_d_sum_of_sqr",
i.e. the derivative of sum-of-squares.
.SH DESCRIPTION
This function performs training of a feedforward net using the conjugate
gradients algorithm. The computation of Hessian is avoided by calculating
directly the product between a direction and Hessian trough a finite
difference approach which require just two error gradients
(see function ann_BP_VHess). The error gradient is calculated
across all training patterns at once given in x (respectively t).
.SH
EXAMPLES
See examples/ann/encoder_cc.sce in the home directory of your Scilab
installation for a full example.
.SH SEE ALSO
.TP
ANN
general overview of ANN toolbox.
.TP
ANN_GEN
general support functions.
.TP
ANN_FF
general description of supported feedforward nets and algorithms.
.TP
ann_FF_init
initialization of weights hypermatrix.
.TP
ann_FF_run
runs the network.
.TP
ann_FF_grad_BP
computation of error gradient.
.TP
ann_FF_VHess
computation of multiplication between a vector and Hessian.
.SH AUTHOR
Ryurick M. Hristev <r.hristev@phys.canterbury.ac.nz>
