// ART2 network for signal recognition

clear;
rand('seed',0);
lines(40,120);

// 20 points are taken from a period of signal, signals are:
//      sigmoid : 0.4 * sin(x) + 0.5
//      step, first half high at 0.9, second half low at 0.1
//      "saw dent", increasing from 0.1 to 0.9
//      triangular from 0.1 to 0.9 and back
//      constant at 0.5

x = [0.5000, 0.9, 0.10, 0.10, 0.5;
     0.6236, 0.9, 0.14, 0.18, 0.5;
     0.7351, 0.9, 0.18, 0.26, 0.5;
     0.8236, 0.9, 0.22, 0.34, 0.5;
     0.8804, 0.9, 0.26, 0.42, 0.5;
     0.9000, 0.9, 0.30, 0.50, 0.5;
     0.8804, 0.9, 0.34, 0.58, 0.5;
     0.8236, 0.9, 0.38, 0.66, 0.5;
     0.7351, 0.9, 0.42, 0.74, 0.5;
     0.6236, 0.9, 0.46, 0.82, 0.5;
     0.5000, 0.1, 0.50, 0.90, 0.5;
     0.3764, 0.1, 0.54, 0.82, 0.5;
     0.2649, 0.1, 0.58, 0.74, 0.5;
     0.1764, 0.1, 0.62, 0.66, 0.5;
     0.1196, 0.1, 0.66, 0.58, 0.5;
     0.1000, 0.1, 0.70, 0.50, 0.5;
     0.1196, 0.1, 0.74, 0.42, 0.5;
     0.1764, 0.1, 0.78, 0.34, 0.5;
     0.2649, 0.1, 0.82, 0.26, 0.5;
     0.3764, 0.1, 0.86, 0.18, 0.5];

// test data reduce amplitude by 30%
x_test = (x - 0.5) * 0.7 + 0.5;

// network constants
a = 0.2;
b = 0.2;
c = 0.001;
d = 0.999;
rho = 0.99;

// network size
N = size(x,'r');
K = 8;

// pattern number
T = size(x,'c');

// piecewise linear contrast enhancement
alpha = 0.1;

// initialize weights
W1 = zeros(N,K);
W2 = rand(K,N) / (sqrt(K) * (1-d));

// hold calibration and test results
calibrated = zeros(K,1);
win_test = zeros(K,1);

// -------------------------------------------------------------------
// train
for t = 1 : T
  // initialize inactive neurons (all active)
  inactive = ones(K,1);
  // initialize and do a propagation trough F_1, find F_2 winner
  w = x(:,t);
  s = w / sqrt(w' * w);
  v = clean(s,alpha);
  u = v / sqrt(v' * v);
  p = u;
  q = p / sqrt(p' * p);

  // find F_2 winner
  [tmp,winner] = max(inactive .* (W2 * p));

  // reiterate to stabilize
  for i = 1 : 10
    w = x(:,t) + a * u;
    s = w / sqrt(w' * w);
    v = clean(s,alpha) + b * clean(q,alpha);
    u = v / sqrt(v' * v);
    p = u + d * W1(:,winner);
    q = p / sqrt(p' * p);
  end;

  // calculate reset output
  r = (u + c * p) / (sqrt(u' * u) + c * sqrt(p' * p));
  match = sqrt(r' * r);
  while rho > match do ...
    // inactivate current winner
    inactive(winner) = 0;
    // restart with current input
    w = x(:,t);
    s = w / sqrt(w' * w);
    v = clean(s,alpha);
    u = v / sqrt(v' * v);
    p = u;
    q = p / sqrt(p' * p);

    // find F_2 winner
    [tmp,winner] = max(inactive .* (W2 * p));
    
    // reiterate to stabilize
    for i = 1 : 10
      w = x(:,t) + a * u;
      s = w / sqrt(w' * w);
      v = clean(s,alpha) + b * clean(q,alpha);
      u = v / sqrt(v' * v);
      p = u + d * W1(:,winner);
      q = p / sqrt(p' * p);
    end;

    // calculate reset output
    r = (u + c * p) / (sqrt(u' * u) + c * sqrt(p' * p));
    match = sqrt(r' * r);
  end;

  // winner found, calibrate and update weights
  calibrated(t) = winner;
  W1(:,winner) = u / (1-d);
  W2(winner,:) = W1(:,winner)';
end;

// -------------------------------------------------------------------
// test
for t = 1 : T
  // initialize inactive neurons (all active)
  inactive = ones(K,1);
  // initialize and do a propagation trough F_1, find F_2 winner
  w = x_test(:,t);
  s = w / sqrt(w' * w);
  v = clean(s,alpha);
  u = v / sqrt(v' * v);
  p = u;
  q = p / sqrt(p' * p);

  // find F_2 winner
  [tmp,winner] = max(inactive .* (W2 * p));

  // reiterate to stabilize
  for i = 1 : 10
    w = x_test(:,t) + a * u;
    s = w / sqrt(w' * w);
    v = clean(s,alpha) + b * clean(q,alpha);
    u = v / sqrt(v' * v);
    p = u + d * W1(:,winner);
    q = p / sqrt(p' * p);
  end;

  // calculate reset output
  r = (u + c * p) / (sqrt(u' * u) + c * sqrt(p' * p));
  match = sqrt(r' * r);
  while rho > match do ...
    // inactivate current winner
    inactive(winner) = 0;
    // restart with current input
    w = x_test(:,t);
    s = w / sqrt(w' * w);
    v = clean(s,alpha);
    u = v / sqrt(v' * v);
    p = u;
    q = p / sqrt(p' * p);

    // find F_2 winner
    [tmp,winner] = max(inactive .* (W2 * p));
    
    // reiterate to stabilize
    for i = 1 : 10
      w = x_test(:,t) + a * u;
      s = w / sqrt(w' * w);
      v = clean(s,alpha) + b * clean(q,alpha);
      u = v / sqrt(v' * v);
      p = u + d * W1(:,winner);
      q = p / sqrt(p' * p);
    end;

    // calculate reset output
    r = (u + c * p) / (sqrt(u' * u) + c * sqrt(p' * p));
    match = sqrt(r' * r);
  end;
  
  // save winner
  win_test(t) = winner;
end;

calibrated
win_test
