// SciLab 2.4 script

//WARNING THIS SCRIPT DOES NOT WORK UNDER SCILAB 2.6

// SOM/Kohonen bidimensional feature map:
// K neurons arranged in a square LxL layout (K = L^2),
// mapping a set of 6-dimensional vectors


/////////////////////////////////////////////////////////////////////
// Depending over constants chosen and due to the fact that the
// learning set is randomly chosen, the learning may not be always
// succesfull.
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// Variables and setup

clear;

printf("PLEASE WAIT ...",1)

// starting point for random numbers always the same (for comparison purposes)
rand('seed',0);

// number of neurons:
//   per side
L = 9;
//   per total
K = L^2;
// there are only 64 patterns but 81 neurons,
// this is to allow for some maneuvering room

// T = 150, total number of training sessions
T = 150;

// initialize data, generate x = 0, 1, ... ,63 in binary format
x = [ ones(1,1)  .*. ([0,1] .*. ([1,1] .*. ([1,1] .*. ([1,1] .*. ([1,1] .*. [1,1])))));
      ones(1,2)  .*. ([0,1] .*. ([1,1] .*. ([1,1] .*. ([1,1] .*.  [1,1]))));
      ones(1,4)  .*. ([0,1] .*. ([1,1] .*. ([1,1] .*.  [1,1])));
      ones(1,8)  .*. ([0,1] .*. ([1,1] .*.  [1,1]));
      ones(1,16) .*. ([0,1] .*.  [1,1]);
      ones(1,32) .*.  [0,1] ]';

// size of input vectors
N = size(x,'c');
// number of input patterns
P = size(x,'r');

// keep a copy for later calibration
// (patterns will be shuffled between epochs)
x_orig = x;

// initialize weights
W = rand(K,N);

// stop function
tau = 1;
tau_rate = 0.9997;

// distance const. in lateral feedback function
d = 4.2^2;
d_rate = 0.997^2;
// feedback height
h = 0.8;

// this vector helps compute distances
delta = linspace(1,L,L)';

// train net

// --- network training ---
for t = 1 : T;
  // randomly shuffle patterns between epochs
  my_rand = ceil(P * rand(P,1));
  for p = 1 : P
    temp = x(my_rand(p),:);
    x(my_rand(p),:) = x(p,:);
    x(p,:) = temp;
  end;

  for p = 1 : P
    // - winner first -
    // find distances
    // first differences of type [x1(p) - weight1, ...,  x6(p) - weight6]
    dist = (ones(K,1) * x(p,:)) - W;
    // and then squared Euclidean distances
    dist_sqr = sum(dist .* dist, 'c');
  
    // and winner k
    [winner_dist,k] = min(dist_sqr);
    // x,y coordinates (network topology wise)
    k_x = modulo(k-1,L) + 1;
    k_y = (k - k_x)/L + 1;
  
    // - stop function -
    tau = tau * tau_rate;

    // - lateral feedback function -
    d = d * d_rate;
  
    // "x" and "y" set of squared distances (reduced by current d)
    dd_x = delta - k_x * ones(L,1);
    d_x = dd_x .* dd_x / d;
    dd_y = delta - k_y * ones(L,1);
    d_y = dd_y .* dd_y / d;
  
    // lateral feedback (note the order of multiplication!)
    psi = h * (exp(- d_y) .*. exp(- d_x));
  
    // update weights
    W = W + (tau * psi * ones(1,N)) .* (ones(K,1) * x(p,:) - W);
  end;
end;

// reset x to original for calibration
x = x_orig;

// calibrate: find winner for all innput vectors

// initialize calibrated matrix
calibrated_net = - ones(L,L);

for p = 1 : P
  // - winner first -
  // find distances
  // first differences of type [x1(p) - weight1, ...,  x6(p) - weight6]
  dist = (ones(K,1) * x(p,:)) - W;
  // and then squared Euclidean distances
  dist_sqr = sum(dist .* dist, 'c');
  
  // and winner k
  [winner_dist,k] = min(dist_sqr);
  // x,y coordinates (network topology wise)
  k_x = modulo(k-1,L) + 1;
  k_y = (k - k_x)/L + 1;
  
  // update calibration matrix
  calibrated_net(k_y,k_x) = p-1;
end;

calibrated_net_string = string(calibrated_net);
for i = 1 : L
  for j = 1 : L
    if calibrated_net_string(i,j) == '-1' then
      calibrated_net_string(i,j) = '\cdot';
    end;
  end;
end;

// print calibrated network
printf("Calibrated Network (\cdot are inactive neurons):",1)
calibrated_net_string

// find calibrated_net in octal
// first digit
calibrated_net_oct = ...
    string(int((calibrated_net + 8 * ones(L,L))/8) - ones(L,L));
// second digit
calibrated_net_oct = calibrated_net_oct + ...
    string(modulo(calibrated_net,8));

for i = 1 : L
  for j = 1 : L
    if calibrated_net_oct(i,j) == '-1-1' then
      calibrated_net_oct(i,j) = '\cdot';
    end;
  end;
end;

// print calibrated_net in octal
printf("Calibrated Network octal (\cdot are inactive neurons):",1)
calibrated_net_oct

// find differences in bits between lateral neighbors
horizontal_neighbor = - ones(L,L-1);
for i = 1 : L
  for j = 1 : L-1
    if calibrated_net(i,j) > -1 & calibrated_net(i,j+1) > -1 then
      horizontal_neighbor(i,j) = ...
          sum(abs(x_orig(calibrated_net(i,j)+1,:) ...
                  - x_orig(calibrated_net(i,j+1)+1,:)), 'c');
    else
      horizontal_neighbor(i,j) = -1;
    end;
  end;
end;

horizontal_neighbor = string(horizontal_neighbor);
for i = 1 : L
  for j = 1 : L-1
    if horizontal_neighbor(i,j) == '-1' then
      horizontal_neighbor(i,j) = '\cdot';
    end;
  end;
end;

printf("Horizontal neighbor difference, count bits (\cdot means no neighbor):",1)
horizontal_neighbor

// find differences in bits between vertical neighbors
vertical_neighbor = - ones(L-1,L);
for i = 1 : L-1
  for j = 1 : L
    if calibrated_net(i,j) > -1 & calibrated_net(i+1,j) > -1 then
      vertical_neighbor(i,j) = ...
          sum(abs(x_orig(calibrated_net(i,j)+1,:) ...
                  - x_orig(calibrated_net(i+1,j)+1,:)), 'c');
    else
      vertical_neighbor(i,j) = -1;
    end;
  end;
end;

vertical_neighbor = string(vertical_neighbor);
for i = 1 : L-1
  for j = 1 : L
    if vertical_neighbor(i,j) == '-1' then
      vertical_neighbor(i,j) = '\cdot';
    end;
  end;
end;

printf("Vertical neighbor difference, count bits (\cdot means no neighbor):",1)
vertical_neighbor

// find differences in bits between diagonal neighbors
// (upper-left to lower-right)
diag1_neighbor = - ones(L-1,L-1);
for i = 1 : L-1
  for j = 1 : L-1
    if calibrated_net(i,j) > -1 & calibrated_net(i+1,j+1) > -1 then
      diag1_neighbor(i,j) = ...
          sum(abs(x_orig(calibrated_net(i,j)+1,:) ...
                  - x_orig(calibrated_net(i+1,j+1)+1,:)), 'c');
    else
      diag1_neighbor(i,j) = -1;
    end;
  end;
end;

diag1_neighbor = string(diag1_neighbor);
for i = 1 : L-1
  for j = 1 : L-1
    if diag1_neighbor(i,j) == '-1' then
      diag1_neighbor(i,j) = '\cdot';
    end;
  end;
end;

printf("Diagonal neighbor difference, count bits",1);
printf("(upper-left <-> lower-right, \cdot means no neighbor):",1)
diag1_neighbor

// find differences in bits between diagonal neighbors
// (lower-left to upper-right)
diag2_neighbor = - ones(L-1,L-1);
for i = 1 : L-1
  for j = 1 : L-1
    if calibrated_net(i+1,j) > -1 & calibrated_net(i,j+1) > -1 then
      diag2_neighbor(i,j) = ...
          sum(abs(x_orig(calibrated_net(i+1,j)+1,:) ...
                  - x_orig(calibrated_net(i,j+1)+1,:)), 'c');
    else
      diag2_neighbor(i,j) = -1;
    end;
  end;
end;

diag2_neighbor = string(diag2_neighbor);
for i = 1 : L-1
  for j = 1 : L-1
    if diag2_neighbor(i,j) == '-1' then
      diag2_neighbor(i,j) = '\cdot';
    end;
  end;
end;

printf("Diagonal neighbor difference, count bits",1);
printf("(lower-left <-> upper-right, \cdot means no neighbor):",1)
diag2_neighbor
